# Copyright (c) 2014 Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class _Iterator(object):
    """Base Iterator

    This iterator is not meant to be used outside
    the scope of this package. The iterator gets
    a dictionary as returned by a listing endpoint.

    Subclasses of this base class determine the key
    to iterate over, as well as the means of creating
    the objects contained within.

    If there are no objects left to return, the iterator
    will try to load more by following the `next` rel link
    type.

    The iterator raises a StopIteration exception if the server
    doesn't return more objects after a `next-page` call.

    :param client: The client instance used by the queue
    :type client: `v1.Client`
    :param listing_response: Response returned by the listing call
    :type listing_response: Dict
    """
    def __init__(self, client, listing_response, iter_key, create_function):
        self._client = client
        self._iter_key = iter_key
        self._create_function = create_function

        self._links = []
        self._stream = False
        self._listing_response = listing_response

        # NOTE(flaper87): Simple hack to
        # re-use the iterator for get_many_messages
        # and message listing.
        if isinstance(listing_response, dict):
            self._links = listing_response.get('links', [])
            self._listing_response = listing_response[self._iter_key]

    def __iter__(self):
        return self

    def get_iterables(self, iterables):
        self._links = iterables['links']
        self._listing_response = iterables[self._iter_key]

    def stream(self, enabled=True):
        """Make this `_Iterator` a stream iterator.

        Since `_Iterator`'s default is to *not* stream,
        this method's default value is to *stream* data
        from the server. That is, unless explicitly specified
        this method will enable make this iterator a stream
        iterator.

        :param enabled: Whether streaming should be
                        enabled or not.
        :type enabled: bool
        """
        self._stream = enabled
        return self

    def _next_page(self):
        for link in self._links:
            if link['rel'] == 'next':
                # NOTE(flaper87): We already have the
                # ref for the next set of messages, lets
                # just follow it.
                iterables = self._client.follow(link['href'])

                # NOTE(flaper87): Since we're using
                # `.follow`, the empty result will
                # be None. Consider making the API
                # return an empty dict for consistency.
                if iterables:
                    self.get_iterables(iterables)
                    return
        raise StopIteration

    def __next__(self):
        try:
            args = self._listing_response.pop(0)
        except IndexError:
            if not self._stream:
                raise StopIteration

            self._next_page()
            return self.next()

        return self._create_function(args)

    # NOTE(flaper87): Py2K support
    next = __next__



# Copyright (c) 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import mock

from zaqarclient.queues.v1 import iterator
from zaqarclient.tests.queues import base
from zaqarclient.transport import response


class QueuesV1_1FlavorUnitTest(base.QueuesTestBase):

    def test_flavor_create(self):
        pool_list = ['pool1', 'pool2']
        flavor_data = {'pool_list': pool_list}

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:

            resp = response.Response(None, None)
            send_method.return_value = resp

            # NOTE(flaper87): This will call
            # ensure exists in the client instance
            # since auto_create's default is True
            flavor = self.client.flavor('tasty', **flavor_data)
            self.assertEqual('tasty', flavor.name)

    def test_flavor_get(self):
        flavor_data = {'name': 'test'}

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:

            resp = response.Response(None, json.dumps(flavor_data))
            send_method.return_value = resp

            # NOTE(flaper87): This will call
            # ensure exists in the client instance
            # since auto_create's default is True
            flavor = self.client.flavor('test')
            flavor1 = flavor.get()
            self.assertEqual('test', flavor1['name'])

    def test_flavor_update(self):
        pool_list1 = ['pool1', 'pool2']
        pool_list2 = ['pool3', 'pool4']
        flavor_data = {'pool_list': pool_list1}
        updated_data = {'pool_list': pool_list2}

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:
            resp = response.Response(None, json.dumps(updated_data))
            send_method.return_value = resp

            flavor = self.client.flavor('tasty', **flavor_data)
            flavor.update({'pool_list': pool_list2})
            self.assertEqual(pool_list2, flavor.pool_list)

    def test_flavor_list(self):
        returned = {
            'links': [{
                'rel': 'next',
                'href': '/v1.1/flavors?marker=6244-244224-783'
            }],
            'flavors': [{
                'name': 'tasty'
            }]
        }

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:

            resp = response.Response(None, json.dumps(returned))
            send_method.return_value = resp

            flavor_var = self.client.flavors(limit=1)
            self.assertIsInstance(flavor_var, iterator._Iterator)
            self.assertEqual(1, len(list(flavor_var)))

    def test_flavor_delete(self):
        pool_list = ['pool1', 'pool2']
        flavor_data = {'pool_list': pool_list}

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:

            resp = response.Response(None, None)
            resp_data = response.Response(None, json.dumps(flavor_data))
            send_method.side_effect = iter([resp_data, resp])
            # NOTE(flaper87): This will call
            # ensure exists in the client instance
            # since auto_create's default is True
            flavor = self.client.flavor('tasty', **flavor_data)
            flavor.delete()

            # NOTE(flaper87): Nothing to assert here,
            # just checking our way down to the transport
            # doesn't crash.


class QueuesV1_1FlavorFunctionalTest(base.QueuesTestBase):

    def test_flavor_create(self):
        pool_data = {'uri': 'mongodb://127.0.0.1:27017',
                     'weight': 10,
                     'flavor': 'tasty'}
        pool = self.client.pool('stomach', **pool_data)
        self.addCleanup(pool.delete)
        pool_list = ['stomach']
        flavor_data = {'pool_list': pool_list}
        flavor = self.client.flavor('tasty', **flavor_data)
        self.addCleanup(flavor.delete)

        self.assertEqual('tasty', flavor.name)
        self.assertEqual(pool_list, flavor.pool_list)

    def test_flavor_get(self):
        pool_data = {'weight': 10,
                     'flavor': 'tasty',
                     'uri': 'mongodb://127.0.0.1:27017'}
        pool = self.client.pool('stomach', **pool_data)
        self.addCleanup(pool.delete)

        pool_list = ['stomach']
        flavor_data = {'pool_list': pool_list}
        flavor = self.client.flavor('tasty', **flavor_data)
        resp_data = flavor.get()
        self.addCleanup(flavor.delete)

        self.assertEqual('tasty', resp_data['name'])

    def test_flavor_update(self):
        pool_data = {'weight': 10,
                     'uri': 'mongodb://127.0.0.1:27017',
                     'flavor': 'tasty'}

        pool = self.client.pool('stomach', **pool_data)
        self.addCleanup(pool.delete)

        pool_list = ['stomach']
        flavor_data = {'pool_list': pool_list}
        flavor = self.client.flavor('tasty', **flavor_data)
        self.addCleanup(flavor.delete)

    def test_flavor_list(self):
        pool_data = {'uri': 'mongodb://127.0.0.1:27017',
                     'weight': 10,
                     'flavor': 'test_flavor'}

        pool = self.client.pool('stomach', **pool_data)
        self.addCleanup(pool.delete)

        pool_list = ['stomach']
        flavor_data = {'pool_list': pool_list}
        flavor = self.client.flavor("test_flavor", **flavor_data)
        self.addCleanup(flavor.delete)

        flavors = self.client.flavors()
        self.assertIsInstance(flavors, iterator._Iterator)
        self.assertEqual(1, len(list(flavors)))

    def test_flavor_delete(self):
        pool_data = {'uri': 'mongodb://127.0.0.1:27017',
                     'weight': 10,
                     'flavor': 'tasty'}
        pool = self.client.pool('stomach', **pool_data)
        self.addCleanup(pool.delete)

        pool_list = ['stomach']
        flavor_data = {'pool_list': pool_list}
        flavor = self.client.flavor('tasty', **flavor_data)
        flavor.delete()


class QueuesV2FlavorUnitTest(QueuesV1_1FlavorUnitTest):
    pass


class QueuesV2FlavorFunctionalTest(QueuesV1_1FlavorFunctionalTest):
    pass



# Copyright (c) 2015 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from zaqarclient.queues.v2 import core


class Subscription(object):

    def __init__(self, client, queue_name, subscriber=None, ttl=60, id=None,
                 auto_create=True, **kwargs):
        self.client = client

        self.id = id
        self.queue_name = queue_name
        self.subscriber = subscriber
        self.ttl = ttl
        self.options = kwargs.get('options', {})
        self.age = kwargs.get('age')
        self.confirmed = kwargs.get('confirmed')

        if auto_create:
            self.ensure_exists()

    def ensure_exists(self):
        """Ensures subscription exists

        This method is not race safe, the subscription could've been deleted
        right after it was called.
        """
        req, trans = self.client._request_and_transport()

        if not self.id and self.subscriber:
            subscription_data = {'subscriber': self.subscriber,
                                 'ttl': self.ttl,
                                 'options': self.options
                                 }
            subscription = core.subscription_create(trans, req,
                                                    self.queue_name,
                                                    subscription_data)

            if subscription and 'subscription_id' in subscription:
                self.id = subscription['subscription_id']

        if self.id:
            sub = core.subscription_get(trans, req, self.queue_name, self.id)
            self.subscriber = sub.get('subscriber')
            self.ttl = sub.get('ttl')
            self.options = sub.get('options')
            self.age = sub.get('age')
            self.confirmed = sub.get('confirmed')

    def update(self, subscription_data):
        req, trans = self.client._request_and_transport()
        core.subscription_update(trans, req, self.queue_name,
                                 self.id, subscription_data)

        for key, value in subscription_data.items():
            setattr(self, key, value)

    def delete(self):
        req, trans = self.client._request_and_transport()
        core.subscription_delete(trans, req, self.queue_name, self.id)


def create_object(parent):
    return lambda kwargs: Subscription(parent, kwargs.pop('source'),
                                       subscriber=kwargs.pop('subscriber'),
                                       ttl=kwargs.pop('ttl'),
                                       id=kwargs.pop('id'),
                                       age=kwargs.pop('age'),
                                       confirmed=kwargs.pop('confirmed'),
                                       auto_create=False,
                                       **kwargs)



# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
from oslo_utils import netutils

from zaqarclient.queues import client
from zaqarclient.tests import base
from zaqarclient.tests.transport import dummy


MY_IP = netutils.get_my_ipv4()


class QueuesTestBase(base.TestBase):

    transport_cls = dummy.DummyTransport
    url = 'http://%s:8888' % MY_IP
    version = 1

    def setUp(self):
        super(QueuesTestBase, self).setUp()
        self.transport = self.transport_cls(self.conf)

        self.client = client.Client(self.url, self.version,
                                    self.conf)

        mocked_transport = mock.Mock(return_value=self.transport)
        self.client._get_transport = mocked_transport
        self.queue = self.client.queue(1, auto_create=False)



# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import zaqarclient
from zaqarclient import errors
from zaqarclient.queues import client
from zaqarclient.tests import base


class TestClient(base.TestBase):

    def test_get_instance(self):
        version = list(client._CLIENTS.keys())[0]
        cli = client.Client('http://example.com',
                            version, {})
        self.assertIsInstance(cli,
                              client._CLIENTS[version])

    def test_version_failure(self):
        self.assertRaises(errors.ZaqarError,
                          client.Client,
                          'http://example.org',
                          -1, {})

    def test_module_version(self):
        self.assertTrue(hasattr(zaqarclient, '__version__'))



# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json


class Response(object):
    """Common response class for Zaqarclient.

    All `zaqarclient.transport.base.Transport` implementations
    will return this to the higher level API which will then build
    an object out of it.

    :param request: The request sent to the server.
    :type: `zaqarclient.transport.request.Request`
    :param content: Response's content
    :type: `six.string_types`
    :param headers: Optional headers returned in the response.
    :type: dict
    :param status_code: Optional status_code returned in the response.
    :type: `int`
    """

    __slots__ = ('request', 'content', 'headers', 'status_code',
                 '_deserialized')

    def __init__(self, request, content, headers=None, status_code=None):
        self.request = request
        self.content = content
        self.headers = headers or {}
        self.status_code = status_code

        self._deserialized = None

    @property
    def deserialized_content(self):
        try:
            if not self._deserialized and self.content:
                self._deserialized = json.loads(self.content)
            return self._deserialized
        except ValueError as ex:
            print("Response is not a JSON object.", ex)
        return None



# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from zaqarclient.queues.v1 import queues
from zaqarclient.queues.v2 import claim as claim_api
from zaqarclient.queues.v2 import core
from zaqarclient.queues.v2 import message


class Queue(queues.Queue):

    message_module = message

    def signed_url(self, paths=None, ttl_seconds=None, methods=None):
        req, trans = self.client._request_and_transport()
        return core.signed_url_create(trans, req, self._name, paths=paths,
                                      ttl_seconds=ttl_seconds, methods=methods)

    def subscriptions(self, detailed=False, marker=None, limit=20):
        return self.client.subscriptions(queue_name=self._name,
                                         detailed=detailed,
                                         marker=marker,
                                         limit=limit)

    def metadata(self, new_meta=None, force_reload=False):
        """Get metadata and return it

        :param new_meta: A dictionary containing
            an updated metadata object. If present
            the queue metadata will be updated in
            remote server. If the new_meta is empty,
            the metadata object will be cleared.
        :type new_meta: `dict`
        :param force_reload: Whether to ignored the
            cached metadata and reload it from the
            server.
        :type force_reload: `bool`

        :returns: The queue metadata.
        """
        req, trans = self.client._request_and_transport()

        # TODO(flaper87): Cache with timeout
        if new_meta is None and self._metadata and not force_reload:
            return self._metadata
        else:
            self._metadata = core.queue_get(trans, req, self._name)

        if new_meta is not None:
            temp_metadata = self._metadata.copy()
            changes = []
            for key, value in new_meta.items():
                # If key exists, replace it's value.
                if self._metadata.get(key, None) is not None:
                    changes.append({'op': 'replace',
                                    'path': '/metadata/%s' % key,
                                    'value': value})
                    temp_metadata.pop(key)
                # If not, add the new key.
                else:
                    changes.append({'op': 'add',
                                    'path': '/metadata/%s' % key,
                                    'value': value})
            # For the keys which are not included in the new metadata, remove
            # them.
            for key, value in temp_metadata.items():
                changes.append({'op': 'remove',
                                'path': '/metadata/%s' % key})

            self._metadata = core.queue_update(trans, req, self._name,
                                               metadata=changes)

        return self._metadata

    def purge(self, resource_types=None):
        req, trans = self.client._request_and_transport()
        core.queue_purge(trans, req, self._name,
                         resource_types=resource_types)

    def claim(self, id=None, ttl=None, grace=None,
              limit=None):
        return claim_api.Claim(self, id=id, ttl=ttl, grace=grace, limit=limit)


def create_object(parent):
    return lambda args: Queue(parent, args["name"], href=args.get("href"),
                              metadata=args.get("metadata"), auto_create=False)



# Copyright (c) 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import mock

from zaqarclient.queues.v1 import iterator
from zaqarclient.tests.queues import base
from zaqarclient.transport import response


class QueuesV1PoolUnitTest(base.QueuesTestBase):

    def test_pool_create(self):
        pool_data = {'weight': 10,
                     'uri': 'sqlite://'}

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:

            resp = response.Response(None, None)
            send_method.return_value = resp

            # NOTE(flaper87): This will call
            # ensure exists in the client instance
            # since auto_create's default is True
            pool = self.client.pool('test', **pool_data)
            self.assertEqual('test', pool.name)
            self.assertEqual(10, pool.weight)

    def test_pool_get(self):
        pool_data = {'weight': 10,
                     'name': 'test',
                     'uri': 'sqlite://',
                     'options': {}}

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:

            resp = response.Response(None, json.dumps(pool_data))
            send_method.return_value = resp

            # NOTE(flaper87): This will call
            # ensure exists in the client instance
            # since auto_create's default is True

            pool = self.client.pool('test')
            pool1 = pool.get()
            self.assertEqual('test', pool1['name'])
            self.assertEqual(10, pool1['weight'])

    def test_pool_update(self):
        pool_data = {'weight': 10,
                     'uri': 'sqlite://',
                     'options': {}}
        updated_data = {'weight': 20,
                        'uri': 'sqlite://',
                        'options': {}}

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:

            resp = response.Response(None, json.dumps(updated_data))
            send_method.return_value = resp

            pool = self.client.pool('test', **pool_data)
            pool.update({'weight': 20})
            self.assertEqual(20, pool.weight)

    def test_pool_list(self):
        returned = {
            'links': [{
                'rel': 'next',
                'href': '/v1.1/pools?marker=6244-244224-783'
            }],
            'pools': [{
                'name': 'stomach',
                'weight': 20,
                'uri': 'sqlite://',
                'options': {}
            }]
        }

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:

            resp = response.Response(None, json.dumps(returned))
            send_method.return_value = resp

            pools_var = self.client.pools(limit=1)
            self.assertIsInstance(pools_var, iterator._Iterator)
            self.assertEqual(1, len(list(pools_var)))

    def test_pool_delete(self):
        pool_data = {'weight': 10,
                     'uri': 'sqlite://',
                     'options': {}}

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:

            resp = response.Response(None, None)
            resp_data = response.Response(None, json.dumps(pool_data))
            send_method.side_effect = iter([resp_data, resp])
            # NOTE(flaper87): This will call
            # ensure exists in the client instance
            # since auto_create's default is True
            pool = self.client.pool('test', **pool_data)
            pool.delete()

            # NOTE(flaper87): Nothing to assert here,
            # just checking our way down to the transport
            # doesn't crash.


class QueuesV1_1PoolFunctionalTest(base.QueuesTestBase):

    def test_pool_get(self):
        pool_data = {'weight': 10,
                     'uri': 'mongodb://127.0.0.1:27017'}

        pool = self.client.pool('FuncTestPool', **pool_data)
        resp_data = pool.get()
        self.addCleanup(pool.delete)

        self.assertEqual('FuncTestPool', resp_data['name'])
        self.assertEqual(10, resp_data['weight'])
        self.assertEqual('mongodb://127.0.0.1:27017', resp_data['uri'])

    def test_pool_create(self):
        pool_data = {'weight': 10,
                     'uri': 'mongodb://127.0.0.1:27017'}

        pool = self.client.pool('FuncTestPool', **pool_data)
        self.addCleanup(pool.delete)
        self.assertEqual('FuncTestPool', pool.name)
        self.assertEqual(10, pool.weight)

    def test_pool_update(self):
        pool_data = {'weight': 10,
                     'uri': 'mongodb://127.0.0.1:27017'}

        pool = self.client.pool('FuncTestPool', **pool_data)
        self.addCleanup(pool.delete)
        pool.update({'weight': 20})
        self.assertEqual(20, pool.weight)

    def test_pool_list(self):
        pool_data = {'weight': 10,
                     'uri': 'mongodb://127.0.0.1:27017'}
        pool = self.client.pool('FuncTestPool', **pool_data)
        self.addCleanup(pool.delete)

        pools = self.client.pools()
        self.assertIsInstance(pools, iterator._Iterator)
        self.assertEqual(1, len(list(pools)))

    def test_pool_delete(self):
        pool_data = {'weight': 10,
                     'uri': 'mongodb://127.0.0.1:27017'}

        pool = self.client.pool('FuncTestPool', **pool_data)
        pool.delete()


class QueuesV2PoolUnitTest(QueuesV1PoolUnitTest):
    pass


class QueuesV2PoolFunctionalTest(QueuesV1_1PoolFunctionalTest):
    pass



# Copyright 2014 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

import ddt

from zaqarclient.queues import client
from zaqarclient.tests.queues import base
from zaqarclient.transport import errors
from zaqarclient.transport import http

VERSIONS = [2]


@ddt.ddt
class TestClient(base.QueuesTestBase):
    transport_cls = http.HttpTransport
    url = 'http://127.0.0.1:8888/v2'
    version = VERSIONS[0]

    @ddt.data(*VERSIONS)
    def test_transport(self, version):
        cli = client.Client('http://example.com',
                            version, {"auth_opts": {'backend': 'noauth'}})
        self.assertIsNotNone(cli.transport())

    @ddt.data(*VERSIONS)
    def test_ping_ok(self, version):
        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:
            send_method.return_value = None
            self.assertTrue(self.client.ping())

    @ddt.data(*VERSIONS)
    def test_ping_bad(self, version):
        def raise_error(*args, **kwargs):
            raise errors.ServiceUnavailableError()

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:
            send_method.side_effect = raise_error
            self.assertFalse(self.client.ping())
